@@ -446,6 +446,7 @@ proc Affopts { device } {
   global timeelapsed
   global usefirst
   global wireframe
+  global showfps
   global use3d
   global trackmouse
 
@@ -492,6 +493,7 @@ proc Affopts { device } {
   if { $timeelapsed == 1 } {append linecommand "-timeelapsed "}
   if { $usefirst == 1 } {append linecommand "-usefirst "}
   if { $wireframe == 1 } {append linecommand "-wireframe "}
+  if { $showfps == 1 } {append linecommand "-showfps "}
   if { $use3d == 1 } {append linecommand "-use3d "}
   if { $trackmouse == 1 } {append linecommand "-trackmouse "}
   if { $nolock == 1 } {append linecommand "-nolock "}
@@ -598,6 +600,7 @@ $CHECK add check -label "sound" -variable sound
 $CHECK add check -label "timeelapsed" -variable timeelapsed
 $CHECK add check -label "usefirst" -variable usefirst
 $CHECK add check -label "wireframe" -variable wireframe
+$CHECK add check -label "showfps" -variable showfps
 $CHECK add check -label "use3d" -variable use3d
 $CHECK add check -label "trackmouse" -variable trackmouse
 
@@ -668,6 +671,7 @@ decay\
 deco\
 demon\
 dilemma\
+dragon\
 discrete\
 drift\
 euler2d\
@@ -680,6 +684,7 @@ flow\
 forest\
 galaxy\
 gears\
+glplanet\
 goop\
 grav\
 helix\
@@ -714,9 +719,9 @@ munch\
 nose\
 pacman\
 penrose\
-pentominoes\
 petal\
 pipes\
+polyominoes\
 puzzle\
 pyro\
 qix\
@@ -724,8 +729,10 @@ roll\
 rotor\
 rubik\
 sballs\
+scooter\
 shape\
 sierpinski\
+sierpinski3d\
 skewb\
 slip\
 solitare\
