@@ -119,10 +119,11 @@ ModeSpecOpt ant_opts =
 
 #ifdef USE_MODULES
 ModStruct   ant_description =
-{"ant", "init_ant", "draw_ant", "release_ant",
- "refresh_ant", "init_ant", NULL, &ant_opts,
- 1000, -3, 40000, -12, 64, 1.0, "",
- "Shows Langton's and Turk's generalized ants", 0, NULL};
+{(char *) "ant",
+ (char *) "init_ant", (char *) "draw_ant", (char *) "release_ant",
+ (char *) "refresh_ant", (char *) "init_ant", NULL, &ant_opts,
+ 1000, -3, 40000, -12, 64, 1.0, (char *) "",
+ (char *) "Shows Langton's and Turk's generalized ants", 0, NULL};
 
 #endif
 
@@ -1087,7 +1088,7 @@ init_ant(ModeInfo * mi)
 		ap->ncols = nccols / 2;
 		ap->nrows = 2 * (ncrows / 4);
 		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs / 2;
-		ap->yb = (ap->height - ap->ys * (ncrows / 2) * 2) / 2 + ap->ys;
+		ap->yb = (ap->height - ap->ys * (ncrows / 2) * 2) / 2 + ap->ys - 2;
 		for (i = 0; i < 6; i++) {
 			ap->shape.hexagon[i].x = (ap->xs - 1) * hexagonUnit[i].x;
 			ap->shape.hexagon[i].y = ((ap->ys - 1) * hexagonUnit[i].y / 2) * 4 / 3;
