@@ -79,7 +79,7 @@ static const char sccsid[] = "@(#)bat.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_bat
 
 ModeSpecOpt bat_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   bat_description =
