@@ -57,7 +57,7 @@ static const char sccsid[] = "@(#)coral.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_coral
 
 ModeSpecOpt coral_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   coral_description =
