@@ -60,7 +60,7 @@ static const char sccsid[] = "@(#)deco.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_deco
 
 ModeSpecOpt deco_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   deco_description =
