@@ -68,7 +68,8 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
 
 #ifdef MODE_flow
 
-ModeSpecOpt flow_opts = { 0, NULL, 0, NULL, NULL };
+ModeSpecOpt flow_opts =
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   flow_description = {
