@@ -172,7 +172,7 @@ ModStruct   atlantis_description =
 
 #endif
 
-static atlantisstruct *atlantis = NULL;
+static atlantisstruct *atlantis = (atlantisstruct *) NULL;
 
 static void
 parse_image_data(ModeInfo *mi)
@@ -423,7 +423,7 @@ init_atlantis(ModeInfo * mi)
 						sizeof (fishRec))) == NULL) {
 			/* free everything up to now */
 			(void) free((void *) atlantis);
-			atlantis = NULL;
+			atlantis = (atlantisstruct *) NULL;
 			return;
 		}
 	}
@@ -511,7 +511,7 @@ release_atlantis(ModeInfo * mi)
 			}
 		}
 		(void) free((void *) atlantis);
-		atlantis = NULL;
+		atlantis = (atlantisstruct *) NULL;
 	}
 	FreeAllGL(mi);
 }
