@@ -440,7 +440,7 @@ static float MaterialGray6[] =
 static float MaterialGray7[] =
 {0.7, 0.7, 0.7, 1.0};
 
-static rubikstruct *rubik = NULL;
+static rubikstruct *rubik = (rubikstruct *) NULL;
 static GLuint objects = 0;
 
 
@@ -1767,7 +1767,7 @@ shuffle(ModeInfo * mi)
 	if (rp->storedmoves < 0) {
 		if (rp->moves != NULL)
 			(void) free((void *) rp->moves);
-		rp->moves = NULL;
+		rp->moves = (RubikMove *) NULL;
 		rp->storedmoves = NRAND(-rp->storedmoves) + 1;
 	}
 	if ((rp->storedmoves) && (rp->moves == NULL))
@@ -1876,16 +1876,16 @@ free_rubik(rubikstruct *rp)
 	for (i = 0; i < MAXFACES; i++)
 		if (rp->cubeLoc[i] != NULL) {
 			(void) free((void *) rp->cubeLoc[i]);
-			rp->cubeLoc[i] = NULL;
+			rp->cubeLoc[i] = (RubikLoc *) NULL;
 		}
 	for (i = 0; i < MAXORIENT; i++)
 		if (rp->rowLoc[i] != NULL) {
 			(void) free((void *) rp->rowLoc[i]);
-			rp->rowLoc[i] = NULL;
+			rp->rowLoc[i] = (RubikLoc *) NULL;
 		}
 	if (rp->moves != NULL) {
 		(void) free((void *) rp->moves);
-		rp->moves = NULL;
+		rp->moves = (RubikMove *) NULL;
 	}
 }
 
@@ -1907,7 +1907,7 @@ release_rubik(ModeInfo * mi)
 			}
 		}
 		(void) free((void *) rubik);
-		rubik = NULL;
+		rubik = (rubikstruct *) NULL;
 	}
 	FreeAllGL(mi);
 }
