@@ -162,7 +162,7 @@ typedef struct {
 } sballsstruct;
 
 /* array of sballsstruct indexed by screen number */
-static sballsstruct *sballs = NULL;
+static sballsstruct *sballs = (sballsstruct *) NULL;
 
 /* lights */
 static float LightAmbient[]=   { 1.0f, 1.0f, 1.0f, 1.0f };
@@ -800,7 +800,7 @@ void release_sballs(ModeInfo * mi)
 	    }
 	}
 	(void) free((void *) sballs);
-	sballs = NULL;
+	sballs = (sballsstruct *) NULL;
     }
     if (mode_font != None)
     {
