@@ -8,7 +8,7 @@ static const char sccsid[] = "@(#)skewb.c	5.00 2000/11/01 xlockmore";
 
 #undef DEBUG_LISTS
 #undef HACK /* I am just doing experiments here to figure it out */
-#define HACK
+/* #define HACK */
 
 /*-
  * Permission to use, copy, modify, and distribute this software and its
@@ -386,6 +386,7 @@ static int  faceToRotate[MAXFACES][MAXORIENT] =
 	{3, 0, 1, 4}
 };
 
+#ifdef HACK
 static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
 {
 	{
@@ -521,6 +522,7 @@ static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
 			{4, 2, 3}}
 	}
 };
+#endif
 
 typedef struct {
 	GLint       WindH, WindW;
@@ -584,7 +586,7 @@ static float MaterialGray6[] =
 static float MaterialGray7[] =
 {0.7, 0.7, 0.7, 1.0};
 
-static skewbstruct *skewb = NULL;
+static skewbstruct *skewb = (skewbstruct *) NULL;
 static GLuint objects = 0;
 
 
@@ -784,7 +786,7 @@ static void
 draw_cubit(ModeInfo * mi,
 	   int back, int front, int left, int right, int bottom, int top)
 {
-	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
+	/* skewbstruct *sp = &skewb[MI_SCREEN(mi)]; */
 	int         mono = MI_IS_MONO(mi);
 
 	if (back != NO_FACE) {
@@ -921,11 +923,12 @@ draw_cubit(ModeInfo * mi,
 	}
 }
 
+#ifdef HACK
 static void
 draw_facit(ModeInfo * mi,
 	   int back, int front, int left, int right, int bottom, int top)
 {
-	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
+	/* skewbstruct *sp = &skewb[MI_SCREEN(mi)]; */
 	int         mono = MI_IS_MONO(mi);
 
 	if (back != NO_FACE) {
@@ -965,6 +968,7 @@ draw_facit(ModeInfo * mi,
 		glEnd();
 	}
 }
+#endif
 
 static Bool
 draw_cube(ModeInfo * mi)
@@ -976,18 +980,17 @@ draw_cube(ModeInfo * mi)
 	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
 	SkewbLoc  slice;
 	GLfloat     rotatestep;
-	int         i, j, k;
+	/* int         i, j, k; */
 
 	if (sp->movement.face == NO_FACE) {
 		slice.face = NO_FACE;
 		slice.rotation = NO_ROTATION;
-	} else {
-#ifdef FIXME
-		convertMove(sp, sp->movement, &slice);
-#else
-		;
-#endif
 	}
+#ifdef FIXME
+	 else {
+		convertMove(sp, sp->movement, &slice);
+	}
+#endif
 	rotatestep = (slice.rotation == CCW) ? sp->rotatestep : -sp->rotatestep;
 
 
@@ -1165,6 +1168,7 @@ rotateFace(skewbstruct * sp, int face, int direction)
 	/* DrawDiamond(face); */ 
 }
 
+#ifdef HACK
 static      Boolean
 checkMoveDir(int position1, int position2, int *direction)
 {
@@ -1189,6 +1193,7 @@ checkMoveDir(int position1, int position2, int *direction)
 	*direction += 2 * MAXORIENT;
 	return True;
 }
+#endif
 
 static void
 moveSkewb(skewbstruct * sp, int face, int direction, int position)
@@ -1271,6 +1276,7 @@ evalmovement(ModeInfo * mi, SkewbMove movement)
 
 }
 
+#ifdef HACK
 static      Bool
 compare_moves(skewbstruct * sp, SkewbMove move1, SkewbMove move2, Bool opp)
 {
@@ -1291,6 +1297,7 @@ compare_moves(skewbstruct * sp, SkewbMove move1, SkewbMove move2, Bool opp)
 #endif
 	return False;
 }
+#endif
 
 static Bool
 shuffle(ModeInfo * mi)
@@ -1309,7 +1316,7 @@ shuffle(ModeInfo * mi)
 	if (sp->storedmoves < 0) {
 		if (sp->moves != NULL)
 			(void) free((void *) sp->moves);
-		sp->moves = NULL;
+		sp->moves = (SkewbMove *) NULL;
 		sp->storedmoves = NRAND(-sp->storedmoves) + 1;
 	}
 	if ((sp->storedmoves) && (sp->moves == NULL))
@@ -1402,7 +1409,7 @@ free_skewb(skewbstruct *sp)
 {
 	if (sp->moves != NULL) {
 		(void) free((void *) sp->moves);
-		sp->moves = NULL;
+		sp->moves = (SkewbMove *) NULL;
 	}
 }
 
@@ -1424,7 +1431,7 @@ release_skewb(ModeInfo * mi)
 			}
 		}
 		(void) free((void *) skewb);
-		skewb = NULL;
+		skewb = (skewbstruct *) NULL;
         }
 	FreeAllGL(mi);
 }
