@@ -73,13 +73,13 @@ static const char sccsid[] = "@(#)sproingiewrap.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_sproingies
 
 ModeSpecOpt sproingies_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   sproingies_description =
 {"sproingies", "init_sproingies", "draw_sproingies", "release_sproingies",
  "refresh_sproingies", "init_sproingies", NULL, &sproingies_opts,
- 1000, 5, 0, 400, 4, 1.0, "",
+ 1000, 5, 0, 400, 64, 1.0, "",
  "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL};
 
 #endif
@@ -112,7 +112,7 @@ typedef struct {
 	int         init;
 } sproingiesstruct;
 
-static sproingiesstruct *sproingies = NULL;
+static sproingiesstruct *sproingies = (sproingiesstruct *) NULL;
 
 static Display *swap_display;
 static Window swap_window;
@@ -245,7 +245,7 @@ release_sproingies(ModeInfo * mi)
 		}
 
 		(void) free((void *) sproingies);
-		sproingies = NULL;
+		sproingies = (sproingiesstruct *) NULL;
 	}
 	FreeAllGL(mi);
 }
