@@ -217,7 +217,7 @@ static void t3d_anim_rotatexy(text3dstruct * tp);
 static void t3d_anim_rotateyz(text3dstruct * tp);
 static void t3d_anim_frequency(text3dstruct * tp);
 static void t3d_anim_amplitude(text3dstruct * tp);
-static char *fontfile = NULL;
+static char *fontfile = (char *) NULL;
 
 #ifdef __cplusplus
 }
@@ -252,7 +252,7 @@ static char * anim_names[] =
 	(char *) "RotateYZ",
 	(char *) "Frequency",	/* needs -rot_frequency /= 0.0 */
 	(char *) "Amplitude",	/* and   -rot_amplitude /= 0.0 */
-	NULL
+	(char *) NULL
 };
 
 static int anims=sizeof anim_array / sizeof anim_array[0] ;
@@ -678,7 +678,7 @@ static void
     {
 	int ch = (unsigned char) c_text[i];
 
-#if ((MESA_MAJOR_VERSION > 3 ) || (( MESA_MAJOR_VERSION == 3 ) && (MESA_MINOR_VERSION > 0 )))
+#if ((XMESA_MAJOR_VERSION > 3 ) || (( XMESA_MAJOR_VERSION == 3 ) && (XMESA_MINOR_VERSION > 0 )))
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
 	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
@@ -745,7 +745,7 @@ static void
     for (i = 0; i < text_length; ++i)
     {
 
-#if ((MESA_MAJOR_VERSION > 3 ) || (( MESA_MAJOR_VERSION == 3 ) && (MESA_MINOR_VERSION > 0 )))
+#if ((XMESA_MAJOR_VERSION > 3 ) || (( XMESA_MAJOR_VERSION == 3 ) && (XMESA_MINOR_VERSION > 0 )))
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
 	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
@@ -847,7 +847,7 @@ static void
     for (i = 0; i < text_length; ++i)
     {
 
-#if ((MESA_MAJOR_VERSION > 3 ) || (( MESA_MAJOR_VERSION == 3 ) && (MESA_MINOR_VERSION > 0 )))
+#if ((XMESA_MAJOR_VERSION > 3 ) || (( XMESA_MAJOR_VERSION == 3 ) && (XMESA_MINOR_VERSION > 0 )))
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
 	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
@@ -992,7 +992,7 @@ static void
     for (i = 0; i < text_length; ++i)
     {
 
-#if ((MESA_MAJOR_VERSION > 3 ) || (( MESA_MAJOR_VERSION == 3 ) && (MESA_MINOR_VERSION > 0 )))
+#if ((XMESA_MAJOR_VERSION > 3 ) || (( XMESA_MAJOR_VERSION == 3 ) && (XMESA_MINOR_VERSION > 0 )))
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
 	glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
@@ -1232,7 +1232,7 @@ void
 		delete tp->face;
 	}
 	(void) free((void *) text3d);
-	text3d = NULL;
+	text3d = (text3dstruct *) NULL;
     }
 #ifdef USE_BLANK
     else
