@@ -99,7 +99,7 @@ static const char sccsid[] = "@(#)goop.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_goop
 
 ModeSpecOpt goop_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   goop_description =
