@@ -50,7 +50,7 @@ static const char sccsid[] = "@(#)image.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_image
 
 ModeSpecOpt image_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   image_description =
