@@ -2,7 +2,7 @@
 /* loop --- Chris Langton's self-producing loops */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)loop.c	5.01 2000/03/15 xlockmore";
 
 #endif
 
@@ -22,6 +22,9 @@ static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 15-Mar-2001: Added some flaws, random blue wall spots, to liven it up.
+ *              This mod seems to expose a bug where hexagons are erased
+ *              for no apparent reason.
  * 01-Nov-2000: Allocation checks
  * 16-Jun-2000: Fully coded the hexagonal rules.  (Rules that end up in
  *              state zero were not bothered with since a calloc was used
@@ -35,11 +38,11 @@ static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
  *       xlock -mode loop -neighbors 6 -size 5 -delay 1 -count 540 -nolock
  * 18-Oct-1998: Started creating a hexagon version.
  *              It proved not that difficult because I used Langton's Loop
- *              as a guide, hexagon have more neighbors so there is more
+ *              as a guide, hexagons have more neighbors so there is more
  *              freedom to program, and the loop will has six sides to
  *              store its genes (data).
- *              (Soon after this a triangular version with neighbors = 6 but
- *              was unsuccessful).
+ *              (Soon after this a triangular version with neighbors = 6
+ *              was attempted but was unsuccessful).
  * 10-May-1997: Compatible with xscreensaver
  * 15-Nov-1995: Coded from Chris Langton's Self-Reproduction in Cellular
  *              Automata Physica 10D 135-144 1984, also used wire.c as a
@@ -73,8 +76,8 @@ static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
    The data spinning around could be viewed as both its DNA and its internal
    clock.  The program can be initalized to have the loop spin both ways...
    but only one way around will work for a given rule.  An open question (at
-   least to me): Is handedness a requirement for life?  Here there is
-   handedness at both the initial condition and the transition rule.
+   least to me): Is handedness a requirement for (artificial) life?  Here
+   there is handedness at both the initial condition and the transition rule.
  */
 
 #ifdef STANDALONE
@@ -83,6 +86,7 @@ static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
 #define HACK_DRAW draw_loop
 #define loop_opts xlockmore_opts
 #define DEFAULTS "*delay: 100000 \n" \
+ "*count: -5 \n" \
  "*cycles: 1600 \n" \
  "*size: -12 \n" \
  "*ncolors: 15 \n" \
@@ -126,7 +130,7 @@ ModeSpecOpt loop_opts =
 ModStruct   loop_description =
 {"loop", "init_loop", "draw_loop", "release_loop",
  "refresh_loop", "init_loop", NULL, &loop_opts,
- 100000, 1, 1600, -12, 64, 1.0, "",
+ 100000, 5, 1600, -12, 64, 1.0, "",
  "Shows Langton's self-producing loops", 0, NULL};
 
 #endif
@@ -766,30 +770,26 @@ position_of_neighbor(int dir, int *pcol, int *prow)
 	if (local_neighbors == 6) {
 		switch (dir) {
 			case 0:
-				col = col + 1;
+				col++;
 				break;
 			case 60:
-				if (row & 1)
-					col = col + 1;
-				row = row - 1;
+				col += (row & 1);
+				row--;
 				break;
 			case 120:
-				if (!(row & 1))
-					col = col - 1;
-				row = row - 1;
+				col -= !(row & 1);
+				row--;
 				break;
 			case 180:
-				col = col - 1;
+				col--;
 				break;
 			case 240:
-				if (!(row & 1))
-					col = col - 1;
-				row = row + 1;
+				col -= !(row & 1);
+				row++;
 				break;
 			case 300:
-				if (row & 1)
-					col = col + 1;
-				row = row + 1;
+				col += (row & 1);
+				row++;
 				break;
 			default:
 				(void) fprintf(stderr, "wrong direction %d\n", dir);
@@ -797,16 +797,16 @@ position_of_neighbor(int dir, int *pcol, int *prow)
 	} else {
 		switch (dir) {
 			case 0:
-				col = col + 1;
+				col++;
 				break;
 			case 90:
-				row = row - 1;
+				row--;
 				break;
 			case 180:
-				col = col - 1;
+				col--;
 				break;
 			case 270:
-				row = row + 1;
+				row++;
 				break;
 			default:
 				(void) fprintf(stderr, "wrong direction %d\n", dir);
@@ -1086,6 +1086,51 @@ init_table()
 	return True;
 }
 
+static void
+init_flaw(ModeInfo * mi)
+{
+	loopstruct *lp = &loops[MI_SCREEN(mi)];
+	int a, b;
+
+#define	BLUE 2
+	if (lp->bncols <= 3 || lp->bnrows <= 3)
+		return;
+	a = NRAND(lp->bncols - 3);
+	b = NRAND(lp->bnrows - 3);
+	if (lp->mincol > a)
+		lp->mincol = a;
+	if (lp->minrow > b)
+		lp->minrow = b;
+	if (lp->maxcol < a + 2)
+		lp->maxcol = a + 2;
+	if (lp->maxrow < b + 2)
+		lp->maxrow = b + 2;
+
+	if (local_neighbors == 6) {
+		lp->newcells[b * lp->bncols + a + !(b % 2) ] = BLUE;
+		lp->newcells[b * lp->bncols + a + 1 + !(b % 2)] = BLUE;
+		lp->newcells[(b + 1) * lp->bncols + a] = BLUE;
+		lp->newcells[(b + 1) * lp->bncols + a + 2] = BLUE;
+		lp->newcells[(b + 2) * lp->bncols + a + !(b % 2)] = BLUE;
+		lp->newcells[(b + 2) * lp->bncols + a + 1 + !(b % 2)] = BLUE;
+	} else {
+		int orient = NRAND(4);
+		lp->newcells[lp->bncols * (b + 1) + a + 1] = BLUE;
+		if (orient == 0 || orient == 1) {
+			lp->newcells[lp->bncols * b + a + 1] = BLUE;
+		}
+		if (orient == 1 || orient == 2) {
+			lp->newcells[lp->bncols * (b + 1) + a + 2] = BLUE;
+		}
+		if (orient == 2 || orient == 3) {
+			lp->newcells[lp->bncols * (b + 2) + a + 1] = BLUE;
+		}
+		if (orient == 3 || orient == 0) {
+			lp->newcells[lp->bncols * (b + 1) + a] = BLUE;
+		}
+	}
+}
+
 static void
 init_adam(ModeInfo * mi)
 {
@@ -1110,10 +1155,14 @@ init_adam(ModeInfo * mi)
 			case 0:
 				start.x = (lp->bncols - HEX_ADAM_LOOPX / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPY) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - 1;
-				lp->maxcol = start.x + HEX_ADAM_LOOPX + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
+				if (lp->mincol > start.x - 2)
+					lp->mincol = start.x - 2;
+				if (lp->minrow > start.y - 1)
+					lp->minrow = start.y - 1;
+				if (lp->maxcol < start.x + HEX_ADAM_LOOPX + 1)
+					lp->maxcol = start.x + HEX_ADAM_LOOPX + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPY + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
 				for (j = 0; j < HEX_ADAM_LOOPY; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPX; i++) {
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPY / 2) % 2) ? -j / 2 : -(j + 1) / 2);
@@ -1127,10 +1176,14 @@ init_adam(ModeInfo * mi)
 			case 1:
 				start.x = (lp->bncols - (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - HEX_ADAM_LOOPX;
-				lp->maxcol = start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
+				if (lp->mincol > start.x - 1)
+					lp->mincol = start.x - 1;
+				if (lp->minrow > start.y - HEX_ADAM_LOOPX)
+					lp->minrow = start.y - HEX_ADAM_LOOPX;
+				if (lp->maxcol < start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1)
+					lp->maxcol = start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPY + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
 				for (j = 0; j < HEX_ADAM_LOOPY; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPX; i++) {
 						k = (((lp->bnrows / 2 + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2) % 2)
@@ -1145,10 +1198,14 @@ init_adam(ModeInfo * mi)
 			case 2:
 				start.x = (lp->bncols - HEX_ADAM_LOOPY / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPX) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - 1;
-				lp->maxcol = start.x + HEX_ADAM_LOOPY + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPX + 1;
+				if (lp->mincol > start.x - 2)
+					lp->mincol = start.x - 2;
+				if (lp->minrow > start.y - 1)
+					lp->minrow = start.y - 1;
+				if (lp->maxcol < start.x + HEX_ADAM_LOOPX + 1)
+					lp->maxcol = start.x + HEX_ADAM_LOOPX + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPY + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
 				for (j = 0; j < HEX_ADAM_LOOPX; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPY; i++) {
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPX / 2) % 2) ? -(HEX_ADAM_LOOPX - j - 1) / 2 : -(HEX_ADAM_LOOPX - j) / 2);
@@ -1162,10 +1219,14 @@ init_adam(ModeInfo * mi)
 			case 3:
 				start.x = (lp->bncols - HEX_ADAM_LOOPX / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPY) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - 1;
-				lp->maxcol = start.x + HEX_ADAM_LOOPX + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
+				if (lp->mincol > start.x - 1)
+					lp->mincol = start.x - 1;
+				if (lp->minrow > start.y - 1)
+					lp->minrow = start.y - 1;
+				if (lp->maxcol < start.x + HEX_ADAM_LOOPX + 1)
+					lp->maxcol = start.x + HEX_ADAM_LOOPX + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPY + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
 				for (j = 0; j < HEX_ADAM_LOOPY; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPX; i++) {
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPY / 2) % 2) ? -j / 2 : -(j + 1) / 2);
@@ -1179,10 +1240,14 @@ init_adam(ModeInfo * mi)
 			case 4:
 				start.x = (lp->bncols - (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - HEX_ADAM_LOOPX;
-				lp->maxcol = start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
+				if (lp->mincol > start.x - 1)
+					lp->mincol = start.x - 1;
+				if (lp->minrow > start.y - HEX_ADAM_LOOPX)
+					lp->minrow = start.y - HEX_ADAM_LOOPX;
+				if (lp->maxcol < start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1)
+					lp->maxcol = start.x + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2 + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPY + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPY + 1;
 				for (j = 0; j < HEX_ADAM_LOOPY; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPX; i++) {
 						k = (((lp->bnrows / 2 + (HEX_ADAM_LOOPX + HEX_ADAM_LOOPY) / 2) % 2)
@@ -1197,10 +1262,14 @@ init_adam(ModeInfo * mi)
 			case 5:
 				start.x = (lp->bncols - HEX_ADAM_LOOPY / 2) / 2;
 				start.y = (lp->bnrows - HEX_ADAM_LOOPX) / 2;
-				lp->mincol = start.x - 2;
-				lp->minrow = start.y - 1;
-				lp->maxcol = start.x + HEX_ADAM_LOOPY + 1;
-				lp->maxrow = start.y + HEX_ADAM_LOOPX + 1;
+				if (lp->mincol > start.x - 2)
+					lp->mincol = start.x - 2;
+				if (lp->minrow > start.y - 1)
+					lp->minrow = start.y - 1;
+				if (lp->maxcol < start.x + HEX_ADAM_LOOPY + 1)
+					lp->maxcol = start.x + HEX_ADAM_LOOPY + 1;
+				if (lp->maxrow < start.y + HEX_ADAM_LOOPX + 1)
+					lp->maxrow = start.y + HEX_ADAM_LOOPX + 1;
 				for (j = 0; j < HEX_ADAM_LOOPX; j++) {
 					for (i = 0; i < HEX_ADAM_LOOPY; i++) {
 						k = (((lp->bnrows / 2 + HEX_ADAM_LOOPX / 2) % 2) ? -(HEX_ADAM_LOOPX - j - 1) / 2 : -(HEX_ADAM_LOOPX - j) / 2);
@@ -1214,19 +1283,13 @@ init_adam(ModeInfo * mi)
 		}
 #if DEBUGTEST
 				/* (void) printf ("s %d  s %d \n", start.x, start.y); */
-		(void) printf ("%d %d %d %d %d\t",
+		(void) printf ("%d %d %d %d %d\n",
      start.x + i + ((lp->bnrows / 2 % 2) ? -j / 2 : -(j + 1) / 2) - lp->bx,
 		 start.y + j - lp->by, i, j, hex_self_reproducing_loop[j][i]);
 		/* Draw right away */
 		drawcell(mi, start.x + i + ((lp->bnrows / 2 % 2) ? -j / 2 : -(j + 1) / 2) - lp->bx,
 		 start.y + j - lp->by,
 		 hex_self_reproducing_loop[j][i]);
-#endif
-#if DEBUGTEST
-			 (void) printf ("\n");
-#endif
-#if DEBUGTEST
-			 (void) printf ("\n");
 #endif
   } else {
 		switch (dir) {
@@ -1235,41 +1298,65 @@ init_adam(ModeInfo * mi)
 				start.y = (lp->bnrows - ADAM_LOOPY) / 2;
 				dirx.x = 1, dirx.y = 0;
 				diry.x = 0, diry.y = 1;
-				lp->mincol = start.x, lp->minrow = start.y;
-				lp->maxcol = start.x + ADAM_LOOPX, lp->maxrow = start.y + ADAM_LOOPY;
+				if (lp->mincol > start.x)
+					lp->mincol = start.x;
+				if (lp->minrow > start.y)
+					lp->minrow = start.y;
+				if (lp->maxcol < start.x + ADAM_LOOPX)
+					lp->maxcol = start.x + ADAM_LOOPX;
+				if (lp->maxrow < start.y + ADAM_LOOPY)
+					lp->maxrow = start.y + ADAM_LOOPY;
 				break;
 			case 1:
 				start.x = (lp->bncols + ADAM_LOOPY) / 2;
 				start.y = (lp->bnrows - ADAM_LOOPX) / 2;
 				dirx.x = 0, dirx.y = 1;
 				diry.x = -1, diry.y = 0;
-				lp->mincol = start.x - ADAM_LOOPY, lp->minrow = start.y;
-				lp->maxcol = start.x, lp->maxrow = start.y + ADAM_LOOPX;
+				if (lp->mincol > start.x - ADAM_LOOPY)
+					lp->mincol = start.x - ADAM_LOOPY;
+				if (lp->minrow > start.y)
+					lp->minrow = start.y;
+				if (lp->maxcol < start.x)
+					lp->maxcol = start.x;
+				if (lp->maxrow < start.y + ADAM_LOOPX)
+					lp->maxrow = start.y + ADAM_LOOPX;
 				break;
 			case 2:
 				start.x = (lp->bncols + ADAM_LOOPX) / 2;
 				start.y = (lp->bnrows + ADAM_LOOPY) / 2;
 				dirx.x = -1, dirx.y = 0;
 				diry.x = 0, diry.y = -1;
-				lp->mincol = start.x - ADAM_LOOPX, lp->minrow = start.y - ADAM_LOOPY;
-				lp->maxcol = start.x, lp->maxrow = start.y;
+				if (lp->mincol > start.x - ADAM_LOOPX)
+					lp->mincol = start.x - ADAM_LOOPX;
+				if (lp->minrow > start.y - ADAM_LOOPY)
+					lp->minrow = start.y - ADAM_LOOPY;
+				if (lp->maxcol < start.x)
+					lp->maxcol = start.x;
+				if (lp->maxrow < start.y)
+					lp->maxrow = start.y;
 				break;
 			case 3:
 				start.x = (lp->bncols - ADAM_LOOPY) / 2;
 				start.y = (lp->bnrows + ADAM_LOOPX) / 2;
 				dirx.x = 0, dirx.y = -1;
 				diry.x = 1, diry.y = 0;
-				lp->mincol = start.x, lp->minrow = start.y - ADAM_LOOPX;
-				lp->maxcol = start.x + ADAM_LOOPY, lp->maxrow = start.y;
+				if (lp->mincol > start.x)
+					lp->mincol = start.x;
+				if (lp->minrow > start.y - ADAM_LOOPX)
+					lp->minrow = start.y - ADAM_LOOPX;
+				if (lp->maxcol < start.x + ADAM_LOOPX)
+					lp->maxcol = start.x + ADAM_LOOPX;
+				if (lp->maxrow < start.y)
+					lp->maxrow = start.y;
 				break;
 		}
 		for (j = 0; j < ADAM_LOOPY; j++)
 			for (i = 0; i < ADAM_LOOPX; i++)
-				lp->newcells[(start.y + dirx.y * i + diry.y * j) * lp->bncols +
-					     start.x + dirx.x * i + diry.x * j] =
-					(lp->clockwise) ?
-					self_reproducing_loop[j][ADAM_LOOPX - i - 1] :
-					self_reproducing_loop[j][i];
+			  lp->newcells[lp->bncols * (start.y + dirx.y * i + diry.y * j) +
+			    start.x + dirx.x * i + diry.x * j] =
+			    (lp->clockwise) ?
+			      self_reproducing_loop[j][ADAM_LOOPX - i - 1] :
+			      self_reproducing_loop[j][i];
 #if DEBUG
 		/* Draw right away */
 		drawcell(mi, start.x + dirx.x * i + diry.x * j - lp->bx,
@@ -1385,12 +1472,12 @@ init_loop(ModeInfo * mi)
 	lp->width = MI_WIDTH(mi);
 	lp->height = MI_HEIGHT(mi);
 
-  if (!local_neighbors) {
-    for (i = 0; i < NEIGHBORKINDS; i++) {
-      if (neighbors == plots[i]) {
-        local_neighbors = neighbors;
-        break;
-      }
+	if (!local_neighbors) {
+		for (i = 0; i < NEIGHBORKINDS; i++) {
+			if (neighbors == plots[i]) {
+				local_neighbors = neighbors;
+				break;
+			}
 			if (i == NEIGHBORKINDS - 1) {
 #if 1
 				local_neighbors = plots[NRAND(NEIGHBORKINDS)];
@@ -1480,6 +1567,22 @@ init_loop(ModeInfo * mi)
 		release_loop(mi);
 		return;
 	}
+	lp->mincol = lp->bncols - 1;
+	lp->minrow = lp->bnrows - 1;
+	lp->maxcol = 0; 
+	lp->maxrow = 0;
+#ifndef DELAYDEBUGLOOP
+	{
+		int flaws = MI_COUNT(mi);
+
+		if (flaws < 0)
+			flaws = NRAND(-MI_COUNT(mi) + 1);
+		for (i = 0; i < flaws; i++) {
+			init_flaw(mi);
+		}
+		/* actual flaws might be less since the adam loop done next */
+	}
+#endif
 	init_adam(mi);
 }
 
