@@ -63,7 +63,7 @@ static const char sccsid[] = "@(#)lyapunov.c	5.00 2000/11/01 xlockmore";
 #define FLOATRAND(min,max) ((min)+((double) LRAND()/((double) MAXRAND))*((max)-(min)))
 
 ModeSpecOpt lyapunov_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   lyapunov_description =
