@@ -88,7 +88,7 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_maze
 
 ModeSpecOpt maze_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   maze_description =
