@@ -71,7 +71,7 @@ static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_nose
 
 ModeSpecOpt nose_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   nose_description =
