@@ -61,7 +61,7 @@ static const char sccsid[] = "@(#)pyro.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_pyro
 
 ModeSpecOpt pyro_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   pyro_description =
