@@ -52,7 +52,7 @@ static const char sccsid[] = "@(#)spiral.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_spiral
 
 ModeSpecOpt spiral_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   spiral_description =
