@@ -45,7 +45,7 @@ static const char sccsid[] = "@(#)turtle.c	5.00 2000/11/01 xlockmore";
 #ifdef MODE_turtle
 
 ModeSpecOpt turtle_opts =
-{0, NULL, 0, NULL, NULL};
+{0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
 ModStruct   turtle_description =
