@@ -43,7 +43,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_atlantis
 	{(char *) "atlantis", init_atlantis, draw_atlantis, release_atlantis,
 	 refresh_atlantis, change_atlantis, NULL, &atlantis_opts,
-	 1000, 4, 100, 6000, 64, 1.0, (char *) "",
+	 18000, 4, 100, 6000, 64, 1.0, (char *) "",
 	 (char *) "Shows moving sharks/whales/dolphin", 0, NULL},
 #endif
 #ifdef MODE_ball
@@ -109,7 +109,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_cage
 	{(char *) "cage", init_cage, draw_cage, release_cage,
 	 draw_cage, change_cage, NULL, &cage_opts,
-	 1000, 1, 1, 1, 64, 1.0, (char *) "",
+	 25000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows the Impossible Cage, an Escher-like GL scene", 0, NULL},
 #endif
 #ifdef MODE_clock
@@ -172,6 +172,12 @@ LockStruct  LockProcs[] =
 	 1000, 4096, 2500, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows various discrete maps", 0, NULL},
 #endif
+#ifdef MODE_dragon
+	{(char *) "dragon", init_dragon, draw_dragon, release_dragon,
+	 refresh_dragon, init_dragon, NULL, &dragon_opts,
+	 2000000, 1, 16, -24, 64, 1.0, (char *) "",
+	 (char *) "Shows Deventer's Hexagonal Dragons Maze", 0, NULL},
+#endif
 #ifdef MODE_drift
 	{(char *) "drift", init_drift, draw_drift, release_drift,
 	 refresh_drift, init_drift, NULL, &drift_opts,
@@ -238,6 +244,12 @@ LockStruct  LockProcs[] =
 	 1000, 1, 2, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows GL's gears", 0, NULL},
 #endif
+#ifdef MODE_glplanet
+        {(char *) "glplanet", init_glplanet, draw_glplanet, release_glplanet,
+         draw_glplanet, init_glplanet, NULL, &glplanet_opts,
+         1000, 1, 2, 1, 64, 1.0, (char *) "",
+         (char *) "Animates texture mapped sphere (planet)", 0, NULL},
+#endif
 #ifdef MODE_goop
 	{(char *) "goop", init_goop, draw_goop, release_goop,
 	 init_goop, init_goop, NULL, &goop_opts,
@@ -367,7 +379,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_loop
 	{(char *) "loop", init_loop, draw_loop, release_loop,
 	 refresh_loop, init_loop, NULL, &loop_opts,
-	 100000, 1, 1600, -12, 64, 1.0, (char *) "",
+	 100000, -5, 1600, -12, 64, 1.0, (char *) "",
 	 (char *) "Shows Langton's self-producing loops", 0, NULL},
 #endif
 #ifdef MODE_lyapunov
@@ -406,6 +418,12 @@ LockStruct  LockProcs[] =
 	 1000, 1, 1, 1, 64, 1.0, (char *) "",
     (char *) "Shows Moebius Strip II, an Escher-like GL scene with ants", 0, NULL},
 #endif
+#ifdef MODE_molecule
+       {(char *) "molecule", init_molecule, draw_molecule, release_molecule,
+       draw_molecule, init_molecule, NULL, &molecule_opts,
+       50000, 1, 20, 1, 64, 1.0, (char *) "",
+       (char *) "Draws molecules", 0, NULL},
+#endif
 #ifdef MODE_morph3d
 	{(char *) "morph3d", init_morph3d, draw_morph3d, release_morph3d,
 	 draw_morph3d, change_morph3d, NULL, &morph3d_opts,
@@ -442,12 +460,6 @@ LockStruct  LockProcs[] =
 	 10000, 1, 1, -40, 64, 1.0, (char *) "",
 	 (char *) "Shows Penrose's quasiperiodic tilings", 0, NULL},
 #endif
-#ifdef MODE_pentominoes
-	{(char *) "pentominoes", init_pentominoes, draw_pentominoes, release_pentominoes,
-        refresh_pentominoes, init_pentominoes, NULL, &pentominoes_opts,
-        1000, 1024, 3000, 1, 64, 1.0, (char *) "",
-        (char *) "Shows attempts to place pentominoes into a rectangle", 0, NULL},
-#endif
 #ifdef MODE_petal
 	{(char *) "petal", init_petal, draw_petal, release_petal,
 	 refresh_petal, init_petal, NULL, &petal_opts,
@@ -465,6 +477,12 @@ LockStruct  LockProcs[] =
 	 1000, 2, 5, 500, 64, 1.0, (char *) "",
 	 (char *) "Shows a selfbuilding pipe system", 0, NULL},
 #endif
+#ifdef MODE_polyominoes
+	{(char *) "polyominoes", init_polyominoes, draw_polyominoes, release_polyominoes,
+        refresh_polyominoes, init_polyominoes, NULL, &polyominoes_opts,
+        6000, 1, 8192, 1, 64, 1.0, (char *) "",
+        (char *) "Shows attempts to place polyominoes into a rectangle", 0, NULL},
+#endif
 #ifdef MODE_puzzle
 	{(char *) "puzzle", init_puzzle, draw_puzzle, release_puzzle,
 	 init_puzzle, init_puzzle, NULL, &puzzle_opts,
@@ -507,6 +525,12 @@ LockStruct  LockProcs[] =
 	 10000, 0, 10, 400, 64, 1.0, (char *) "",
 	 (char *) "Shows balls spinning like crazy in GL", 0, NULL},
 #endif
+#ifdef MODE_scooter
+	{(char *) "scooter", init_scooter, draw_scooter, release_scooter,
+	 refresh_scooter, change_scooter, NULL, &scooter_opts,
+	 20000, 24, 3, 100, 64, 1.0, (char *) "",
+	 (char *) "Shows a journey through space tunnel and stars", 0, NULL},
+#endif
 #ifdef MODE_shape
 	{(char *) "shape", init_shape, draw_shape, release_shape,
 	 refresh_shape, init_shape, NULL, &shape_opts,
@@ -519,6 +543,12 @@ LockStruct  LockProcs[] =
 	 400000, 2000, 100, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows Sierpinski's triangle", 0, NULL},
 #endif
+#ifdef MODE_sierpinski3d
+        {(char *) "sierpinski3d", init_gasket, draw_gasket, release_gasket,
+         draw_gasket, init_gasket, NULL, &gasket_opts,
+         1000, 1, 2, 1, 64, 1.0, (char *) "",
+         (char *) "Shows GL's Sierpinski gasket", 0, NULL},
+#endif
 #ifdef MODE_skewb
 	{(char *) "skewb", init_skewb, draw_skewb, release_skewb,
 	 draw_skewb, change_skewb, NULL, &skewb_opts,
@@ -540,7 +570,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_space
 	{(char *) "space", init_space, draw_space, release_space,
 	 refresh_space, init_space, NULL, &space_opts,
-	 10000, 100, 1, 15, 64, 1.0, (char *) "",
+	 10000, 100, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a journey into deep space", 0, NULL},
 #endif
 #ifdef MODE_sphere
