@@ -669,6 +669,10 @@ passwd_invalid(char *passwd)
 }
 #endif
 
+#ifdef OSF1_ENH_SEC
+static int user_oldcrypt, root_oldcrypt;
+#endif
+
 static void
 getCryptedUserPasswd(void)
 {
@@ -763,6 +767,8 @@ getCryptedUserPasswd(void)
 		(void) free((void *) buf);	/* Should never get here */
 	}
 	(void) strcpy(userpass, pw->ufld.fd_encrypt);
+	/* also save encryption algorithm associated with encrypted password */
+	user_oldcrypt = pw->ufld.fd_oldcrypt;
 #else /* !OSF1_ENH_SEC */
 
 	struct passwd *pw;
@@ -905,7 +911,8 @@ getCryptedRootPasswd(void)
 		(void) free((void *) buf);	/* Should never get here */
 	}
 	(void) strcpy(rootpass, pw->ufld.fd_encrypt);
-
+	/* also save encryption algorithm associated with encrypted password */
+	root_oldcrypt = pw->ufld.fd_oldcrypt;
 #else /* !OSF1_ENH_SEC */
 	struct passwd *pw;
 	char       *buf;
@@ -1238,7 +1245,11 @@ checkPasswd(char *buffer)
 	}
 #endif
 	if (!done) {
+#ifdef OSF1_ENH_SEC
+		done = (!strcmp((char *) dispcrypt(buffer, userpass, user_oldcrypt), userpass));
+#else
 		done = (!strcmp((char *) crypt(buffer, userpass), userpass));
+#endif
 		/* userpass is used */
 		if (!*userpass && *buffer)
 			/*
@@ -1251,8 +1262,13 @@ checkPasswd(char *buffer)
 	(void) printf("buffer=%s, encrypt=%s, userpass=%s\n", buffer,
 		      (char *) crypt(buffer, userpass), userpass);
 #endif
-	if (!done && allowroot && !strcmp((char *) crypt(buffer, rootpass),
-			 rootpass)) {
+	if (!done && allowroot &&
+#ifdef OSF1_ENH_SEC
+		!strcmp((char *) dispcrypt(buffer, rootpass, root_oldcrypt), rootpass)
+#else
+		!strcmp((char *) crypt(buffer, rootpass), rootpass)
+#endif
+	) {
 		done = True;	
 		if (!rootpass || !*buffer)
 			/*
