@@ -66,7 +66,7 @@ play_sound(char *filename, Bool verbose)
 #include <prvdef.h>
 #include <ssdef.h>
 
-extern void play_sound_mmov(char* filename, Bool verbose);
+extern void PLAY_SOUND_MMOV(char* filename, Bool verbose);
 #endif
 
 #ifdef USE_VMSPLAY
@@ -129,7 +129,7 @@ play_sound(char* filename, Bool verbose)
 	privs[0] = PRV$M_SYSNAM;
 	pr_status = SYS$SETPRV ( 1, privs, 0, 0 );
 	
-	if ( pr_status == SS$_NORMAL ) play_sound_mmov(filename, verbose);
+	if ( pr_status == SS$_NORMAL ) PLAY_SOUND_MMOV(filename, verbose);
 #endif
      }
 }
