@@ -81,15 +81,7 @@ ERROR ! Define PROGCLASS, HACK_INIT, and HACK_DRAW before including xlockmore.h
 #include "mode.h"
 
 #ifdef USE_GL
-#include <GL/glx.h>
-#ifdef __cplusplus
-  extern "C" {
-#endif
-extern GLXContext *init_GL(ModeInfo *);
-#ifdef __cplusplus
-  }
-#endif
-
+#include "visgl.h"
 #define FreeAllGL(dpy)		/* */
 #endif
 
@@ -189,6 +181,9 @@ XrmOptionDescRec options[100];
 /* THIS MAY SOON BE DEFUNCT, SHOULD WORK NOW THOUGH FOR IMAKE */
 #define HAVE_GLBINDTEXTURE 1
 #if !defined(__cplusplus) && !defined(c_plusplus)
+#ifdef inline
+#undef inline
+#endif
 #define inline
 #endif
 #ifdef AIXV3
@@ -494,7 +489,9 @@ extern void finish(Display * display, Bool closeDisplay);
 #define error xlock_error
 #endif
 extern void error(const char *buf);
+#ifndef DECLARED_GETENV
 extern char * getenv(const char *);
+#endif
 #ifdef __cplusplus
   }
 #endif
