@@ -111,7 +111,9 @@ static Widget ScrolledListModes, PushButtons[PUSHBUTTONS];
  /*Resource string */
 #include "modes.h"
 
+#if 0
 static XmStringCharSet char_set = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;
+#endif
 
 /* some resources of buttons and toggles not really good programming :( */
 
@@ -149,7 +151,7 @@ void exitcallback(Widget w, XtPointer client_data, XtPointer call_data) {
 static void
 f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 {
-	int         i, n;
+	int         i;
 	char        command[500];
 
 #ifdef VMS
@@ -241,7 +243,7 @@ Setup_Widget(Widget father)
 {
 	Arg         args[15];
 	int         i, ac;
-	Widget      Row, PushButtonRow, TogglesRow, ModeLabel, SwitchLabel,exitwidgetB;
+	Widget      PushButtonRow, exitwidgetB;
 	char        string[160];
 	XmString    label_str;
 	XmString    TabXmStr[numprocs];
@@ -270,6 +272,7 @@ Setup_Widget(Widget father)
 	XtSetArg(args[ac], XmNtopWidget, Menuoption); ac++;
 
 #if 0
+	int ModeLabel;
 	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
 
 	XtSetArg(args[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
@@ -362,7 +365,9 @@ Setup_Widget(Widget father)
 		      f_ScrolledListModes, NULL);
 	XtManageChild(ScrolledListModes);
 
-/*	TogglesRow = XmCreateRowColumn(Row, (char *) "TogglesRow", NULL, 0);
+#if 0
+	int TogglesRow, Row;
+	TogglesRow = XmCreateRowColumn(Row, (char *) "TogglesRow", NULL, 0);
 	for (i = 0; i < TOGGLES; i++) {
 #ifndef USE_MB
 		ac = 0;
@@ -379,7 +384,7 @@ Setup_Widget(Widget father)
 	XtManageChild(TogglesRow);
 
 	XtManageChild(Row);
-*/
+#endif
 
 	for (i = 0; i < (int) numprocs; i++) {
 		XmStringFree(TabXmStr[i]);
